@@ -15,7 +15,7 @@ MasonX::Maypole - use Mason as the frontend and view for Maypole version 2
 
 =cut
 
-our $VERSION = 0.34;
+our $VERSION = 0.421;
 
 =head1 SYNOPSIS
 
@@ -115,8 +115,15 @@ sub init {
     $mason_cfg->{in_package}   ||= 'HTML::Mason::Commands';
 
     # this provides dynamic table-name component roots
-    $mason_cfg->{request_class}  = 'MasonX::Request::ExtendedCompRoot';
-    $mason_cfg->{resolver_class} = 'MasonX::Resolver::ExtendedCompRoot';
+    if ( $HTML::Mason::VERSION > 1.2899 )
+    {
+        $mason_cfg->{dynamic_comp_root} = 1;
+    }
+    else
+    {
+        $mason_cfg->{request_class}  = 'MasonX::Request::ExtendedCompRoot';
+        $mason_cfg->{resolver_class} = 'MasonX::Resolver::ExtendedCompRoot';
+    }
 
     $class->mason_ah( MasonX::Maypole::ApacheHandler->new( %{ $mason_cfg } ) );
     
@@ -169,7 +176,7 @@ CROOT:  foreach my $index ( 0 .. $#$comp_roots )
 
     $class->config->masonx->{comp_root} = $comp_roots;
     
-    #warn "Base comp roots: " . YAML::Dump( $comp_roots ) if $class->debug > 2;
+    #use Data::Dumper; warn "Base comp roots: " . Dumper( $comp_roots );
 }
 
 =item parse_args
@@ -199,10 +206,13 @@ phase to generate and send output.
 
 sub send_output {
     my ( $self ) = @_;
-
+    
     # if there was an error, there may already be a report in the output slot,
     # so send it via Apache::MVC
-    return $self->SUPER::send_output if $self->output;
+    if ( $self->output )
+    {
+        return $self->SUPER::send_output if $self->output;
+    }
 
     my $m = eval { $self->mason_ah->prepare_request( $self->ar ) };
 
@@ -234,28 +244,39 @@ sub send_output {
     
     # I think Mason will do this:
     #$self->{ar}->send_http_header;
-
-    # add dynamic comp root for table queries
-    # Changed to using model_class instead of table in 0.219 ( see Maypole::View::Base::paths() 
-    # - ideally, this stuff would go in a paths() method).
-    if ( $self->model_class )
-    {
-        my $model_comp_root = File::Spec->catdir( $self->get_template_root, $self->model_class->moniker );
-        # Even with this ugliness, there might be potential for non-unique prefixes, if a site was 
-        # running multiple Maypole sub-apps all using the same Mason datadir...
-        $m->prefix_comp_root( 'MAYPOLE_MODEL_' . $self->model_class->moniker . "=>$model_comp_root" ) if -d $model_comp_root;
-    }
     
-    #if ( $self->table )
-    #{
-    #    my $table_comp_root = File::Spec->catdir( $self->get_template_root, $self->table );
-    #    $m->prefix_comp_root( "table=>$table_comp_root" ) if -d $table_comp_root;
-    #}
+    my @default_comp_roots = @{ $m->interp->comp_root };
+
+    my $model_comp_root = File::Spec->catdir( $self->get_template_root, $self->model_class->moniker );
+    
+    # Add a dynamic comp root for table queries, if the path exists (often it won't).
+    # See Maypole::View::Base::paths() - maybe this stuff should go in a paths() method.
+    if ( $self->model_class and -d $model_comp_root )
+    {
+        my $label = $self->model_class;
+        $label =~ s/:+/_/g;  
+        
+        if ( $HTML::Mason::VERSION > 1.2899 )
+        {
+            $m->interp->comp_root( [ [ $label => $model_comp_root ], @default_comp_roots ] );
+        }
+        else
+        {
+            $m->prefix_comp_root( "${label}=>$model_comp_root" );
+        }
+    }
     
     warn "Comp roots:\n" . join( "\n", map { "@$_" } @{ $m->interp->comp_root } ) if $self->debug;
 
-    # now generate output
-    $m->exec;
+    # now generate and send output
+    my $status = $m->exec;
+    
+    # maybe saying local $m->interp->comp_root( [ [ $label => $model_comp_root ], @default_comp_roots ] )
+    # would work instead
+    $m->interp->comp_root( [ @default_comp_roots ] ) if $HTML::Mason::VERSION > 1.2899;
+    
+    # Maypole doesn't actually check this status, but for the sake of good form:
+    return $status;
 }
 
 =item get_template_root
@@ -288,6 +309,20 @@ sub get_template_root { $_[0]->config->template_root }
     }
 }
 
+=item get_request
+
+Replaces C<Apache::MVC::get_request>, using C<Apache::Request::instance()> instead 
+of C<Apache::Request::new()> to obtain the APR object. Calling C<new> means Mason 
+and Maypole have different APR objects, and the Mason one doesn't have any POST 
+data.
+
+=cut
+
+sub get_request {
+    my ( $self, $r ) = @_;
+    $self->{ar} = Apache::Request->instance($r); 
+}
+
 =back
 
 =head1 AUTHOR
